Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 55867)
+++ channels/chan_sip.c	(working copy)
@@ -799,11 +799,12 @@
 #define SIP_PAGE2_NOTEXT		(1 << 27)	/*!< 26: Text not supported  */
 #define SIP_PAGE2_TEXTSUPPORT		(1 << 28)	/*!< 27: Global text enable */
 #define SIP_PAGE2_DEBUG_TEXT		(1 << 29)	/*!< 28: Global text debug */
+#define SIP_PAGE2_DISABLE_P2P		(1 << 30)	/*!< 30: Disable Packet to Packet bridging mode for this peer/user */
 
 #define SIP_PAGE2_FLAGS_TO_COPY \
 	(SIP_PAGE2_ALLOWSUBSCRIBE | SIP_PAGE2_ALLOWOVERLAP | SIP_PAGE2_VIDEOSUPPORT | \
 	SIP_PAGE2_T38SUPPORT | SIP_PAGE2_RFC2833_COMPENSATE | SIP_PAGE2_BUGGY_MWI | \
-	SIP_PAGE2_TEXTSUPPORT )
+	SIP_PAGE2_TEXTSUPPORT | SIP_PAGE2_DISABLE_P2P)
 
 /* SIP packet flags */
 #define SIP_PKT_DEBUG		(1 << 0)	/*!< Debug this packet */
@@ -16165,6 +16166,10 @@
 		ast_set_flag(&mask[1], SIP_PAGE2_BUGGY_MWI);
 		ast_set2_flag(&flags[1], ast_true(v->value), SIP_PAGE2_BUGGY_MWI);
 		res = 1;
+	} else if (!strcasecmp(v->name, "disablep2p")) {
+		ast_set_flag(&mask[1], SIP_PAGE2_DISABLE_P2P);
+		ast_set2_flag(&flags[1], ast_true(v->value), SIP_PAGE2_DISABLE_P2P);
+		res = 1;
 	} 
 
 	return res;
@@ -17499,6 +17504,9 @@
 	else if (ast_test_flag(&global_jbconf, AST_JB_FORCED))
 		res = AST_RTP_GET_FAILED;
 
+	if (res == AST_RTP_TRY_PARTIAL && ast_test_flag(&p->flags[1], SIP_PAGE2_DISABLE_P2P))
+		res = AST_RTP_TRY_RAW;
+
 	sip_pvt_unlock(p);
 
 	return res;
Index: include/asterisk/rtp.h
===================================================================
--- include/asterisk/rtp.h	(revision 55867)
+++ include/asterisk/rtp.h	(working copy)
@@ -64,6 +64,8 @@
 	AST_RTP_TRY_PARTIAL,
 	/*! RTP structure exists and native bridge can occur */
 	AST_RTP_TRY_NATIVE,
+	/*! RTP structure exists and partial bridge is disabled */
+	AST_RTP_TRY_RAW,
 };
 
 struct ast_rtp;
Index: main/rtp.c
===================================================================
--- main/rtp.c	(revision 55867)
+++ main/rtp.c	(working copy)
@@ -3422,6 +3422,15 @@
 		return AST_BRIDGE_FAILED_NOWARN;
 	}
 
+	/* If either side cannot do a partial bridge, leave now */
+	if (audio_p0_res == AST_RTP_TRY_RAW || audio_p1_res == AST_RTP_TRY_RAW) {
+			if (option_debug)
+				ast_log(LOG_DEBUG, "Packet2packet bridge explicitly disabled\n");
+			ast_channel_unlock(c0);
+			ast_channel_unlock(c1);
+			return AST_BRIDGE_FAILED_NOWARN;		       
+	}
+
 	/* If either side can only do a partial bridge, then don't try for a true native bridge */
 	if (audio_p0_res == AST_RTP_TRY_PARTIAL || audio_p1_res == AST_RTP_TRY_PARTIAL) {
 		/* In order to do Packet2Packet bridging both sides must be in the same rawread/rawwrite */
